# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.datetime_utils import serialize_datetime
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....claims.types.claim_status import ClaimStatus
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.http_request_validation_error import HttpRequestValidationError
from ....commons.errors.http_request_validations_error import HttpRequestValidationsError
from ....commons.errors.internal_error import InternalError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.encounter_external_id import EncounterExternalId
from ....commons.types.encounter_id import EncounterId
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.internal_error_message import InternalErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.request_validation_error import RequestValidationError
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from ....commons.types.work_queue_id import WorkQueueId
from ....encounters_universal.types.universal_encounter_create import UniversalEncounterCreate
from ....encounters_universal.types.universal_encounter_create_from_pre_encounter import (
    UniversalEncounterCreateFromPreEncounter,
)
from ....encounters_universal.types.universal_encounter_update import UniversalEncounterUpdate
from ....tags.types.tag_id import TagId
from .errors.cash_pay_payer_error import CashPayPayerError
from .errors.encounter_external_id_uniqueness_error import EncounterExternalIdUniquenessError
from .errors.encounter_guarantor_missing_contact_info_error import EncounterGuarantorMissingContactInfoError
from .errors.encounter_patient_control_number_uniqueness_error import EncounterPatientControlNumberUniquenessError
from .errors.encounter_rendering_or_attending_provider_required import EncounterRenderingOrAttendingProviderRequired
from .errors.invalid_tag_names_error import InvalidTagNamesError
from .errors.payer_plan_group_payer_does_not_match_insurance_card_http_error import (
    PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError,
)
from .errors.schema_instance_validation_http_failure import SchemaInstanceValidationHttpFailure
from .types.billable_status_type import BillableStatusType
from .types.cash_pay_payer_error_message import CashPayPayerErrorMessage
from .types.encounter import Encounter
from .types.encounter_create import EncounterCreate
from .types.encounter_create_from_pre_encounter import EncounterCreateFromPreEncounter
from .types.encounter_external_id_uniqueness_error_type import EncounterExternalIdUniquenessErrorType
from .types.encounter_guarantor_missing_contact_info_error_type import EncounterGuarantorMissingContactInfoErrorType
from .types.encounter_owner_of_next_action_type import EncounterOwnerOfNextActionType
from .types.encounter_page import EncounterPage
from .types.encounter_patient_control_number_uniqueness_error_type import (
    EncounterPatientControlNumberUniquenessErrorType,
)
from .types.encounter_rendering_or_attending_provider_required_error import (
    EncounterRenderingOrAttendingProviderRequiredError,
)
from .types.encounter_sort_options import EncounterSortOptions
from .types.encounter_update import EncounterUpdate
from .types.invalid_tag_names_error_type import InvalidTagNamesErrorType
from .types.payer_plan_group_payer_does_not_match_insurance_card_error import (
    PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
)
from .types.responsible_party_type import ResponsiblePartyType
from .types.schema_instance_validation_failure import SchemaInstanceValidationFailure

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV4Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        sort: typing.Optional[EncounterSortOptions] = None,
        page_token: typing.Optional[PageToken] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        primary_payer_names: typing.Optional[str] = None,
        search_term: typing.Optional[str] = None,
        external_id: typing.Optional[EncounterExternalId] = None,
        diagnoses_updated_since: typing.Optional[dt.datetime] = None,
        tag_ids: typing.Optional[typing.Union[TagId, typing.Sequence[TagId]]] = None,
        work_queue_id: typing.Optional[WorkQueueId] = None,
        billable_status: typing.Optional[BillableStatusType] = None,
        responsible_party: typing.Optional[ResponsiblePartyType] = None,
        owner_of_next_action: typing.Optional[EncounterOwnerOfNextActionType] = None,
        patient_external_id: typing.Optional[str] = None,
        include_merged_patient_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EncounterPage]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        claim_status : typing.Optional[ClaimStatus]
            Indicates the current status of an insurance claim within the billing process.

        sort : typing.Optional[EncounterSortOptions]
            Defaults to created_at:desc.

        page_token : typing.Optional[PageToken]

        date_of_service_min : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.

        date_of_service_max : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.

        primary_payer_names : typing.Optional[str]
            Comma delimited string.

        search_term : typing.Optional[str]
            Filter by any of the following fields: encounter_id, claim_id, patient external_id,
            patient date of birth, patient first name, patient last name,
            or encounter external id.

        external_id : typing.Optional[EncounterExternalId]
            Filter to an exact match on encounter external_id, if one exists.

        diagnoses_updated_since : typing.Optional[dt.datetime]
            ISO 8601 timestamp; ideally in UTC (although not required): 2019-08-24T14:15:22Z.

        tag_ids : typing.Optional[typing.Union[TagId, typing.Sequence[TagId]]]
            Filter by name of tags on encounters.

        work_queue_id : typing.Optional[WorkQueueId]

        billable_status : typing.Optional[BillableStatusType]
            Defines if the Encounter is to be billed by Candid to the responsible_party. Examples for when this should be set to NOT_BILLABLE include if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.

        responsible_party : typing.Optional[ResponsiblePartyType]
            Defines the party to be billed with the initial balance owed on the claim. Use SELF_PAY if you intend to bill self pay/cash pay.

        owner_of_next_action : typing.Optional[EncounterOwnerOfNextActionType]
            The party who is responsible for taking the next action on an Encounter, as defined by ownership of open Tasks.

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        include_merged_patient_data : typing.Optional[bool]
            If true and patient_external_id is set, then also include the encounters of all alternative patients.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EncounterPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/encounters/v4",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "claim_status": claim_status,
                "sort": sort,
                "page_token": page_token,
                "date_of_service_min": str(date_of_service_min) if date_of_service_min is not None else None,
                "date_of_service_max": str(date_of_service_max) if date_of_service_max is not None else None,
                "primary_payer_names": primary_payer_names,
                "search_term": search_term,
                "external_id": external_id,
                "diagnoses_updated_since": serialize_datetime(diagnoses_updated_since)
                if diagnoses_updated_since is not None
                else None,
                "tag_ids": tag_ids,
                "work_queue_id": work_queue_id,
                "billable_status": billable_status,
                "responsible_party": responsible_party,
                "owner_of_next_action": owner_of_next_action,
                "patient_external_id": patient_external_id,
                "include_merged_patient_data": include_merged_patient_data,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EncounterPage,
                parse_obj_as(
                    type_=EncounterPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InternalError":
                raise InternalError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InternalErrorMessage,
                        parse_obj_as(
                            type_=InternalErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, encounter_id: EncounterId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Encounter]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Encounter]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/encounters/v4/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_universal(
        self, *, request: UniversalEncounterCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Encounter]:
        """
        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        request : UniversalEncounterCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Encounter]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/encounters/v4/universal",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterPatientControlNumberUniquenessError":
                raise EncounterPatientControlNumberUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterPatientControlNumberUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterPatientControlNumberUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterGuarantorMissingContactInfoError":
                raise EncounterGuarantorMissingContactInfoError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterGuarantorMissingContactInfoErrorType,
                        parse_obj_as(
                            type_=EncounterGuarantorMissingContactInfoErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "CashPayPayerError":
                raise CashPayPayerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        CashPayPayerErrorMessage,
                        parse_obj_as(
                            type_=CashPayPayerErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InvalidTagNamesError":
                raise InvalidTagNamesError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidTagNamesErrorType,
                        parse_obj_as(
                            type_=InvalidTagNamesErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        RequestValidationError,
                        parse_obj_as(
                            type_=RequestValidationError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterRenderingOrAttendingProviderRequired":
                raise EncounterRenderingOrAttendingProviderRequired(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterRenderingOrAttendingProviderRequiredError,
                        parse_obj_as(
                            type_=EncounterRenderingOrAttendingProviderRequiredError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, request: EncounterCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Encounter]:
        """
        Parameters
        ----------
        request : EncounterCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Encounter]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/encounters/v4",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterPatientControlNumberUniquenessError":
                raise EncounterPatientControlNumberUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterPatientControlNumberUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterPatientControlNumberUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterGuarantorMissingContactInfoError":
                raise EncounterGuarantorMissingContactInfoError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterGuarantorMissingContactInfoErrorType,
                        parse_obj_as(
                            type_=EncounterGuarantorMissingContactInfoErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "CashPayPayerError":
                raise CashPayPayerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        CashPayPayerErrorMessage,
                        parse_obj_as(
                            type_=CashPayPayerErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InvalidTagNamesError":
                raise InvalidTagNamesError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidTagNamesErrorType,
                        parse_obj_as(
                            type_=InvalidTagNamesErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        RequestValidationError,
                        parse_obj_as(
                            type_=RequestValidationError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterRenderingOrAttendingProviderRequired":
                raise EncounterRenderingOrAttendingProviderRequired(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterRenderingOrAttendingProviderRequiredError,
                        parse_obj_as(
                            type_=EncounterRenderingOrAttendingProviderRequiredError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_from_pre_encounter_patient_universal(
        self,
        *,
        request: UniversalEncounterCreateFromPreEncounter,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Encounter]:
        """
        Create an encounter from a pre-encounter patient and appointment. This endpoint is intended to be used by consumers who are managing
        patients and appointments in the pre-encounter service and is currently under development. Consumers who are not taking advantage
        of the pre-encounter service should use the standard create endpoint.

        The endpoint will create an encounter from the provided fields, pulling information from the provided patient and appointment objects
        where applicable. In particular, the following fields are populated from the patient and appointment objects:
          - Patient
          - Referring Provider
          - Subscriber Primary
          - Subscriber Secondary
          - Referral Number
          - Responsible Party
          - Guarantor

        Utilizing this endpoint opts you into automatic updating of the encounter when the patient or appointment is updated, assuming the
        encounter has not already been submitted or adjudicated.

        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        request : UniversalEncounterCreateFromPreEncounter

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Encounter]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/encounters/v4/create-from-pre-encounter/universal",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterPatientControlNumberUniquenessError":
                raise EncounterPatientControlNumberUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterPatientControlNumberUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterPatientControlNumberUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        RequestValidationError,
                        parse_obj_as(
                            type_=RequestValidationError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_from_pre_encounter_patient(
        self, *, request: EncounterCreateFromPreEncounter, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Encounter]:
        """
        Create an encounter from a pre-encounter patient and appointment. This endpoint is intended to be used by consumers who are managing
        patients and appointments in the pre-encounter service and is currently under development. Consumers who are not taking advantage
        of the pre-encounter service should use the standard create endpoint.

        The endpoint will create an encounter from the provided fields, pulling information from the provided patient and appointment objects
        where applicable. In particular, the following fields are populated from the patient and appointment objects:
          - Patient
          - Referring Provider
          - Subscriber Primary
          - Subscriber Secondary
          - Referral Number
          - Responsible Party
          - Guarantor

        Utilizing this endpoint opts you into automatic updating of the encounter when the patient or appointment is updated, assuming the
        encounter has not already been submitted or adjudicated.

        Parameters
        ----------
        request : EncounterCreateFromPreEncounter

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Encounter]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/encounters/v4/create-from-pre-encounter",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterPatientControlNumberUniquenessError":
                raise EncounterPatientControlNumberUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterPatientControlNumberUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterPatientControlNumberUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        RequestValidationError,
                        parse_obj_as(
                            type_=RequestValidationError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_universal(
        self,
        encounter_id: EncounterId,
        *,
        request: UniversalEncounterUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Encounter]:
        """
        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        encounter_id : EncounterId

        request : UniversalEncounterUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Encounter]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/encounters/v4/{jsonable_encoder(encounter_id)}/universal",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InvalidTagNamesError":
                raise InvalidTagNamesError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidTagNamesErrorType,
                        parse_obj_as(
                            type_=InvalidTagNamesErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        encounter_id: EncounterId,
        *,
        request: EncounterUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Encounter]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request : EncounterUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Encounter]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/encounters/v4/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InvalidTagNamesError":
                raise InvalidTagNamesError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidTagNamesErrorType,
                        parse_obj_as(
                            type_=InvalidTagNamesErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV4Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        sort: typing.Optional[EncounterSortOptions] = None,
        page_token: typing.Optional[PageToken] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        primary_payer_names: typing.Optional[str] = None,
        search_term: typing.Optional[str] = None,
        external_id: typing.Optional[EncounterExternalId] = None,
        diagnoses_updated_since: typing.Optional[dt.datetime] = None,
        tag_ids: typing.Optional[typing.Union[TagId, typing.Sequence[TagId]]] = None,
        work_queue_id: typing.Optional[WorkQueueId] = None,
        billable_status: typing.Optional[BillableStatusType] = None,
        responsible_party: typing.Optional[ResponsiblePartyType] = None,
        owner_of_next_action: typing.Optional[EncounterOwnerOfNextActionType] = None,
        patient_external_id: typing.Optional[str] = None,
        include_merged_patient_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EncounterPage]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Maximum number of entities per page, defaults to 100.

        claim_status : typing.Optional[ClaimStatus]
            Indicates the current status of an insurance claim within the billing process.

        sort : typing.Optional[EncounterSortOptions]
            Defaults to created_at:desc.

        page_token : typing.Optional[PageToken]

        date_of_service_min : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.

        date_of_service_max : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.

        primary_payer_names : typing.Optional[str]
            Comma delimited string.

        search_term : typing.Optional[str]
            Filter by any of the following fields: encounter_id, claim_id, patient external_id,
            patient date of birth, patient first name, patient last name,
            or encounter external id.

        external_id : typing.Optional[EncounterExternalId]
            Filter to an exact match on encounter external_id, if one exists.

        diagnoses_updated_since : typing.Optional[dt.datetime]
            ISO 8601 timestamp; ideally in UTC (although not required): 2019-08-24T14:15:22Z.

        tag_ids : typing.Optional[typing.Union[TagId, typing.Sequence[TagId]]]
            Filter by name of tags on encounters.

        work_queue_id : typing.Optional[WorkQueueId]

        billable_status : typing.Optional[BillableStatusType]
            Defines if the Encounter is to be billed by Candid to the responsible_party. Examples for when this should be set to NOT_BILLABLE include if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.

        responsible_party : typing.Optional[ResponsiblePartyType]
            Defines the party to be billed with the initial balance owed on the claim. Use SELF_PAY if you intend to bill self pay/cash pay.

        owner_of_next_action : typing.Optional[EncounterOwnerOfNextActionType]
            The party who is responsible for taking the next action on an Encounter, as defined by ownership of open Tasks.

        patient_external_id : typing.Optional[str]
            The patient ID from the external EMR platform for the patient

        include_merged_patient_data : typing.Optional[bool]
            If true and patient_external_id is set, then also include the encounters of all alternative patients.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EncounterPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/encounters/v4",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "claim_status": claim_status,
                "sort": sort,
                "page_token": page_token,
                "date_of_service_min": str(date_of_service_min) if date_of_service_min is not None else None,
                "date_of_service_max": str(date_of_service_max) if date_of_service_max is not None else None,
                "primary_payer_names": primary_payer_names,
                "search_term": search_term,
                "external_id": external_id,
                "diagnoses_updated_since": serialize_datetime(diagnoses_updated_since)
                if diagnoses_updated_since is not None
                else None,
                "tag_ids": tag_ids,
                "work_queue_id": work_queue_id,
                "billable_status": billable_status,
                "responsible_party": responsible_party,
                "owner_of_next_action": owner_of_next_action,
                "patient_external_id": patient_external_id,
                "include_merged_patient_data": include_merged_patient_data,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EncounterPage,
                parse_obj_as(
                    type_=EncounterPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InternalError":
                raise InternalError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InternalErrorMessage,
                        parse_obj_as(
                            type_=InternalErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, encounter_id: EncounterId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Encounter]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Encounter]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/encounters/v4/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_universal(
        self, *, request: UniversalEncounterCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Encounter]:
        """
        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        request : UniversalEncounterCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Encounter]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/encounters/v4/universal",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterPatientControlNumberUniquenessError":
                raise EncounterPatientControlNumberUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterPatientControlNumberUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterPatientControlNumberUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterGuarantorMissingContactInfoError":
                raise EncounterGuarantorMissingContactInfoError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterGuarantorMissingContactInfoErrorType,
                        parse_obj_as(
                            type_=EncounterGuarantorMissingContactInfoErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "CashPayPayerError":
                raise CashPayPayerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        CashPayPayerErrorMessage,
                        parse_obj_as(
                            type_=CashPayPayerErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InvalidTagNamesError":
                raise InvalidTagNamesError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidTagNamesErrorType,
                        parse_obj_as(
                            type_=InvalidTagNamesErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        RequestValidationError,
                        parse_obj_as(
                            type_=RequestValidationError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterRenderingOrAttendingProviderRequired":
                raise EncounterRenderingOrAttendingProviderRequired(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterRenderingOrAttendingProviderRequiredError,
                        parse_obj_as(
                            type_=EncounterRenderingOrAttendingProviderRequiredError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, request: EncounterCreate, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Encounter]:
        """
        Parameters
        ----------
        request : EncounterCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Encounter]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/encounters/v4",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterPatientControlNumberUniquenessError":
                raise EncounterPatientControlNumberUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterPatientControlNumberUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterPatientControlNumberUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterGuarantorMissingContactInfoError":
                raise EncounterGuarantorMissingContactInfoError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterGuarantorMissingContactInfoErrorType,
                        parse_obj_as(
                            type_=EncounterGuarantorMissingContactInfoErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "CashPayPayerError":
                raise CashPayPayerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        CashPayPayerErrorMessage,
                        parse_obj_as(
                            type_=CashPayPayerErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InvalidTagNamesError":
                raise InvalidTagNamesError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidTagNamesErrorType,
                        parse_obj_as(
                            type_=InvalidTagNamesErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        RequestValidationError,
                        parse_obj_as(
                            type_=RequestValidationError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterRenderingOrAttendingProviderRequired":
                raise EncounterRenderingOrAttendingProviderRequired(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterRenderingOrAttendingProviderRequiredError,
                        parse_obj_as(
                            type_=EncounterRenderingOrAttendingProviderRequiredError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_from_pre_encounter_patient_universal(
        self,
        *,
        request: UniversalEncounterCreateFromPreEncounter,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Encounter]:
        """
        Create an encounter from a pre-encounter patient and appointment. This endpoint is intended to be used by consumers who are managing
        patients and appointments in the pre-encounter service and is currently under development. Consumers who are not taking advantage
        of the pre-encounter service should use the standard create endpoint.

        The endpoint will create an encounter from the provided fields, pulling information from the provided patient and appointment objects
        where applicable. In particular, the following fields are populated from the patient and appointment objects:
          - Patient
          - Referring Provider
          - Subscriber Primary
          - Subscriber Secondary
          - Referral Number
          - Responsible Party
          - Guarantor

        Utilizing this endpoint opts you into automatic updating of the encounter when the patient or appointment is updated, assuming the
        encounter has not already been submitted or adjudicated.

        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        request : UniversalEncounterCreateFromPreEncounter

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Encounter]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/encounters/v4/create-from-pre-encounter/universal",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterPatientControlNumberUniquenessError":
                raise EncounterPatientControlNumberUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterPatientControlNumberUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterPatientControlNumberUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        RequestValidationError,
                        parse_obj_as(
                            type_=RequestValidationError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_from_pre_encounter_patient(
        self, *, request: EncounterCreateFromPreEncounter, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Encounter]:
        """
        Create an encounter from a pre-encounter patient and appointment. This endpoint is intended to be used by consumers who are managing
        patients and appointments in the pre-encounter service and is currently under development. Consumers who are not taking advantage
        of the pre-encounter service should use the standard create endpoint.

        The endpoint will create an encounter from the provided fields, pulling information from the provided patient and appointment objects
        where applicable. In particular, the following fields are populated from the patient and appointment objects:
          - Patient
          - Referring Provider
          - Subscriber Primary
          - Subscriber Secondary
          - Referral Number
          - Responsible Party
          - Guarantor

        Utilizing this endpoint opts you into automatic updating of the encounter when the patient or appointment is updated, assuming the
        encounter has not already been submitted or adjudicated.

        Parameters
        ----------
        request : EncounterCreateFromPreEncounter

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Encounter]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/encounters/v4/create-from-pre-encounter",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EncounterPatientControlNumberUniquenessError":
                raise EncounterPatientControlNumberUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterPatientControlNumberUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterPatientControlNumberUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        RequestValidationError,
                        parse_obj_as(
                            type_=RequestValidationError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_universal(
        self,
        encounter_id: EncounterId,
        *,
        request: UniversalEncounterUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Encounter]:
        """
        Notice: The UB-04 Data File, 2025, is copyrighted by American Hospital Association (AHA), Chicago, Illinois.
        No portion of the THE UB-04 Data File, may be reproduced, stored in a retrieval system, or transmitted,
        in any form or by any means, electronic, mechanical, photocopying, recording, or otherwise, without prior
        express, written consent of AHA.

        Parameters
        ----------
        encounter_id : EncounterId

        request : UniversalEncounterUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Encounter]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/encounters/v4/{jsonable_encoder(encounter_id)}/universal",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InvalidTagNamesError":
                raise InvalidTagNamesError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidTagNamesErrorType,
                        parse_obj_as(
                            type_=InvalidTagNamesErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        encounter_id: EncounterId,
        *,
        request: EncounterUpdate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Encounter]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request : EncounterUpdate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Encounter]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/encounters/v4/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Encounter,
                parse_obj_as(
                    type_=Encounter,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EncounterExternalIdUniquenessErrorType,
                        parse_obj_as(
                            type_=EncounterExternalIdUniquenessErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.List[RequestValidationError],
                        parse_obj_as(
                            type_=typing.List[RequestValidationError],  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "SchemaInstanceValidationHttpFailure":
                raise SchemaInstanceValidationHttpFailure(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        SchemaInstanceValidationFailure,
                        parse_obj_as(
                            type_=SchemaInstanceValidationFailure,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "InvalidTagNamesError":
                raise InvalidTagNamesError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        InvalidTagNamesErrorType,
                        parse_obj_as(
                            type_=InvalidTagNamesErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError":
                raise PayerPlanGroupPayerDoesNotMatchInsuranceCardHttpError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayerPlanGroupPayerDoesNotMatchInsuranceCardError,
                        parse_obj_as(
                            type_=PayerPlanGroupPayerDoesNotMatchInsuranceCardError,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...yes_no_indicator.types.yes_no_indicator import YesNoIndicator
from .diagnosis_type_code import DiagnosisTypeCode


class DiagnosisCreateOptional(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Empty string not allowed.
    """

    code_type: typing.Optional[DiagnosisTypeCode] = pydantic.Field(default=None)
    """
    Typically, providers submitting claims to Candid are using ICD-10 diagnosis codes. If you are using ICD-10 codes, the primary diagnosis code listed on the claim should use the ABK code_type. If more than one diagnosis is being submitted on a claim, please use ABF for the rest of the listed diagnoses. If you are using ICD-9 diagnosis codes, use BK and BF for the principal and following diagnosis code(s) respectively.
    """

    code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Empty string not allowed.
    Should be of the appropriate format for the provided `code_type`.
    Must obey the ICD-10 format if an ICD-10 code_type is provided, specifically:
      - Letter
      - Digit
      - Digit or the letter `A` or `B`
      - (Optional) Period `.`
      - Up to 4 (or as few as 0) letters and digits
    """

    present_on_admission_indicator: typing.Optional[YesNoIndicator] = pydantic.Field(default=None)
    """
    For Institutional claims only.
    A "Y" indicates that the onset occurred prior to admission to the hospital.
    An "N" indicates that the onset did NOT occur prior to admission to the hospital.
    A "U" indicates that it is unknown whether the onset occurred prior to admission to the hospital or not.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .intervention_category import InterventionCategory
from .lab_optional import LabOptional
from .medication_optional import MedicationOptional


class InterventionOptional(UniversalBaseModel):
    name: typing.Optional[str] = None
    category: typing.Optional[InterventionCategory] = None
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    "Examples: 'Birth Control LAC', 'Tracking', 'Stress Management', 'Supplement', 'Labs'"
    """

    medication: typing.Optional[MedicationOptional] = pydantic.Field(default=None)
    """
    Required when `type` is `allopathic`.
    """

    labs: typing.Optional[typing.List[LabOptional]] = pydantic.Field(default=None)
    """
    Required when `type` is `tests`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .allocation_target import AllocationTarget
from .balance_earmark import BalanceEarmark


class Allocation(UniversalBaseModel):
    amount_cents: int
    target: AllocationTarget
    earmark: typing.Optional[BalanceEarmark] = pydantic.Field(default=None)
    """
    The active earmark created by this allocation, if any. Only present when this allocation created an earmark for future auto-allocation and the earmark has not been deleted.
    """

    allocated_on: typing.Optional[dt.datetime] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class EligibilityCheckErrorDetails(UniversalBaseModel):
    """
    This object is our fern representation of Stedi's EligbilityCheckError object from their API.
    """

    field: typing.Optional[str] = pydantic.Field(alias="field?", default=None)
    description: typing.Optional[str] = pydantic.Field(alias="description?", default=None)
    location: typing.Optional[str] = pydantic.Field(alias="location?", default=None)
    possible_resolutions: typing.Optional[str] = pydantic.Field(alias="possibleResolutions?", default=None)
    code: typing.Optional[str] = pydantic.Field(alias="code?", default=None)
    followup_action: typing.Optional[str] = pydantic.Field(alias="followupAction?", default=None)

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .condition_code import ConditionCode
from .condition_information_code_qualifier import ConditionInformationCodeQualifier
from .health_care_code_information_new import HealthCareCodeInformationNew


class ConditionInformationNew(HealthCareCodeInformationNew):
    """
    This corresponds to the BG code qualifier value.
    """

    condition_code_qualifier: ConditionInformationCodeQualifier
    condition_code: ConditionCode

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .create_import_invoice_request import CreateImportInvoiceRequest
    from .import_invoice import ImportInvoice
    from .import_invoice_update_request import ImportInvoiceUpdateRequest
    from .import_invoices_page import ImportInvoicesPage
    from .invoice_item_info_update import InvoiceItemInfoUpdate
    from .invoice_item_update_type import InvoiceItemUpdateType
_dynamic_imports: typing.Dict[str, str] = {
    "CreateImportInvoiceRequest": ".create_import_invoice_request",
    "ImportInvoice": ".import_invoice",
    "ImportInvoiceUpdateRequest": ".import_invoice_update_request",
    "ImportInvoicesPage": ".import_invoices_page",
    "InvoiceItemInfoUpdate": ".invoice_item_info_update",
    "InvoiceItemUpdateType": ".invoice_item_update_type",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CreateImportInvoiceRequest",
    "ImportInvoice",
    "ImportInvoiceUpdateRequest",
    "ImportInvoicesPage",
    "InvoiceItemInfoUpdate",
    "InvoiceItemUpdateType",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .key_with_name_already_exists_error import KeyWithNameAlreadyExistsError
    from .schema import Schema
    from .schema_field import SchemaField
    from .schema_get_multi_response import SchemaGetMultiResponse
    from .schema_instance import SchemaInstance
    from .schema_instance_optional import SchemaInstanceOptional
    from .schema_validation_error import (
        SchemaValidationError,
        SchemaValidationError_KeyNameAlreadyExists,
        SchemaValidationError_SchemaNameAlreadyExists,
    )
    from .schema_validation_failure import SchemaValidationFailure
    from .schema_with_name_already_exists_error import SchemaWithNameAlreadyExistsError
_dynamic_imports: typing.Dict[str, str] = {
    "KeyWithNameAlreadyExistsError": ".key_with_name_already_exists_error",
    "Schema": ".schema",
    "SchemaField": ".schema_field",
    "SchemaGetMultiResponse": ".schema_get_multi_response",
    "SchemaInstance": ".schema_instance",
    "SchemaInstanceOptional": ".schema_instance_optional",
    "SchemaValidationError": ".schema_validation_error",
    "SchemaValidationError_KeyNameAlreadyExists": ".schema_validation_error",
    "SchemaValidationError_SchemaNameAlreadyExists": ".schema_validation_error",
    "SchemaValidationFailure": ".schema_validation_failure",
    "SchemaWithNameAlreadyExistsError": ".schema_with_name_already_exists_error",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "KeyWithNameAlreadyExistsError",
    "Schema",
    "SchemaField",
    "SchemaGetMultiResponse",
    "SchemaInstance",
    "SchemaInstanceOptional",
    "SchemaValidationError",
    "SchemaValidationError_KeyNameAlreadyExists",
    "SchemaValidationError_SchemaNameAlreadyExists",
    "SchemaValidationFailure",
    "SchemaWithNameAlreadyExistsError",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        Authorization,
        AuthorizationUnit,
        Contact,
        CoveragesForRelatedCauses,
        DoNotInvoiceReason,
        ExternalProvenance,
        FilingOrder,
        Guarantor,
        InferredPatientMetadata,
        MaritalStatus,
        MutablePatient,
        MutablePatientWithMrn,
        OriginationDetail,
        Patient,
        PatientCoverageSnapshot,
        PatientPage,
        PatientSortField,
        Referral,
        ReferralSource,
        ReferralType,
        ReferralUnit,
        SpecializationCategory,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "Authorization": ".types",
    "AuthorizationUnit": ".types",
    "Contact": ".types",
    "CoveragesForRelatedCauses": ".types",
    "DoNotInvoiceReason": ".types",
    "ExternalProvenance": ".types",
    "FilingOrder": ".types",
    "Guarantor": ".types",
    "InferredPatientMetadata": ".types",
    "MaritalStatus": ".types",
    "MutablePatient": ".types",
    "MutablePatientWithMrn": ".types",
    "OriginationDetail": ".types",
    "Patient": ".types",
    "PatientCoverageSnapshot": ".types",
    "PatientPage": ".types",
    "PatientSortField": ".types",
    "Referral": ".types",
    "ReferralSource": ".types",
    "ReferralType": ".types",
    "ReferralUnit": ".types",
    "SpecializationCategory": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Authorization",
    "AuthorizationUnit",
    "Contact",
    "CoveragesForRelatedCauses",
    "DoNotInvoiceReason",
    "ExternalProvenance",
    "FilingOrder",
    "Guarantor",
    "InferredPatientMetadata",
    "MaritalStatus",
    "MutablePatient",
    "MutablePatientWithMrn",
    "OriginationDetail",
    "Patient",
    "PatientCoverageSnapshot",
    "PatientPage",
    "PatientSortField",
    "Referral",
    "ReferralSource",
    "ReferralType",
    "ReferralUnit",
    "SpecializationCategory",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        Identifier,
        IdentifierBase,
        IdentifierCode,
        IdentifierCreate,
        IdentifierId,
        IdentifierUpdate,
        IdentifierValue,
        IdentifierValue_MedicaidProviderIdentifier,
        IdentifierValue_MedicareProviderIdentifier,
        MedicaidProviderIdentifier,
        MedicareProviderIdentifier,
        UpdatableIdentifier,
        UpdatableIdentifier_Add,
        UpdatableIdentifier_Remove,
        UpdatableIdentifier_Update,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "Identifier": ".types",
    "IdentifierBase": ".types",
    "IdentifierCode": ".types",
    "IdentifierCreate": ".types",
    "IdentifierId": ".types",
    "IdentifierUpdate": ".types",
    "IdentifierValue": ".types",
    "IdentifierValue_MedicaidProviderIdentifier": ".types",
    "IdentifierValue_MedicareProviderIdentifier": ".types",
    "MedicaidProviderIdentifier": ".types",
    "MedicareProviderIdentifier": ".types",
    "UpdatableIdentifier": ".types",
    "UpdatableIdentifier_Add": ".types",
    "UpdatableIdentifier_Remove": ".types",
    "UpdatableIdentifier_Update": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Identifier",
    "IdentifierBase",
    "IdentifierCode",
    "IdentifierCreate",
    "IdentifierId",
    "IdentifierUpdate",
    "IdentifierValue",
    "IdentifierValue_MedicaidProviderIdentifier",
    "IdentifierValue_MedicareProviderIdentifier",
    "MedicaidProviderIdentifier",
    "MedicareProviderIdentifier",
    "UpdatableIdentifier",
    "UpdatableIdentifier_Add",
    "UpdatableIdentifier_Remove",
    "UpdatableIdentifier_Update",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        Contract,
        ContractBase,
        ContractCreate,
        ContractCreateUnion,
        ContractCreateUnion_Institutional,
        ContractCreateUnion_Professional,
        ContractId,
        ContractServiceFacility,
        ContractServiceFacilityBase,
        ContractServiceFacilityId,
        ContractType,
        ContractUnion,
        ContractUnion_Institutional,
        ContractUnion_Professional,
        ContractUpdate,
        ContractUpdateUnion,
        ContractUpdateUnion_Institutional,
        ContractUpdateUnion_Professional,
        ContractWithProvidersUnion,
        ContractWithProvidersUnion_Institutional,
        ContractWithProvidersUnion_Professional,
        ContractingProviderId,
        ContractsPage,
        InstitutionalContract,
        InstitutionalContractCreate,
        InstitutionalContractUpdate,
        ProfessionalContract,
        ProfessionalContractCreate,
        ProfessionalContractUpdate,
        RenderingProviderid,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "Contract": ".types",
    "ContractBase": ".types",
    "ContractCreate": ".types",
    "ContractCreateUnion": ".types",
    "ContractCreateUnion_Institutional": ".types",
    "ContractCreateUnion_Professional": ".types",
    "ContractId": ".types",
    "ContractServiceFacility": ".types",
    "ContractServiceFacilityBase": ".types",
    "ContractServiceFacilityId": ".types",
    "ContractType": ".types",
    "ContractUnion": ".types",
    "ContractUnion_Institutional": ".types",
    "ContractUnion_Professional": ".types",
    "ContractUpdate": ".types",
    "ContractUpdateUnion": ".types",
    "ContractUpdateUnion_Institutional": ".types",
    "ContractUpdateUnion_Professional": ".types",
    "ContractWithProvidersUnion": ".types",
    "ContractWithProvidersUnion_Institutional": ".types",
    "ContractWithProvidersUnion_Professional": ".types",
    "ContractingProviderId": ".types",
    "ContractsPage": ".types",
    "InstitutionalContract": ".types",
    "InstitutionalContractCreate": ".types",
    "InstitutionalContractUpdate": ".types",
    "ProfessionalContract": ".types",
    "ProfessionalContractCreate": ".types",
    "ProfessionalContractUpdate": ".types",
    "RenderingProviderid": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Contract",
    "ContractBase",
    "ContractCreate",
    "ContractCreateUnion",
    "ContractCreateUnion_Institutional",
    "ContractCreateUnion_Professional",
    "ContractId",
    "ContractServiceFacility",
    "ContractServiceFacilityBase",
    "ContractServiceFacilityId",
    "ContractType",
    "ContractUnion",
    "ContractUnion_Institutional",
    "ContractUnion_Professional",
    "ContractUpdate",
    "ContractUpdateUnion",
    "ContractUpdateUnion_Institutional",
    "ContractUpdateUnion_Professional",
    "ContractWithProvidersUnion",
    "ContractWithProvidersUnion_Institutional",
    "ContractWithProvidersUnion_Professional",
    "ContractingProviderId",
    "ContractsPage",
    "InstitutionalContract",
    "InstitutionalContractCreate",
    "InstitutionalContractUpdate",
    "ProfessionalContract",
    "ProfessionalContractCreate",
    "ProfessionalContractUpdate",
    "RenderingProviderid",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...commons.types.claim_id import ClaimId
from ...eras.types.era import Era
from ...service_lines.resources.v_2.types.service_line import ServiceLine
from .claim_status import ClaimStatus


class Claim(UniversalBaseModel):
    """
    Examples
    --------
    import datetime
    import uuid

    from candid.resources.claims import Claim, ClaimStatus
    from candid.resources.commons import (
        DateRangeOptionalEnd,
        FacilityTypeCode,
        ProcedureModifier,
        ServiceLineUnits,
    )
    from candid.resources.eras import Era
    from candid.resources.invoices import Invoice, InvoiceItem, InvoiceStatus
    from candid.resources.service_lines.resources.v_2 import (
        DenialReasonContent,
        ServiceLine,
        ServiceLineAdjustment,
        ServiceLineDenialReason,
        ServiceLineEraData,
        TestResult,
        TestResultType,
    )

    Claim(
        claim_id=uuid.UUID(
            "dd9d7f82-37b5-449d-aa63-26925398335b",
        ),
        status=ClaimStatus.BILLER_RECEIVED,
        clearinghouse="Change Healthcare",
        clearinghouse_claim_id="5BA7C3AB-2BC2-496C-8B10-6CAC73D0729D",
        payer_claim_id="9BB9F259-9756-4F16-8F53-9DBB9F7EB1BB",
        service_lines=[
            ServiceLine(
                created_at=datetime.datetime.fromisoformat(
                    "2023-01-01 00:00:00+00:00",
                ),
                modifiers=[ProcedureModifier.TWENTY_TWO],
                charge_amount_cents=10000,
                allowed_amount_cents=8000,
                insurance_balance_cents=0,
                patient_balance_cents=2000,
                paid_amount_cents=8000,
                patient_responsibility_cents=2000,
                diagnosis_id_zero=uuid.UUID(
                    "4ac84bcd-12f5-4f86-a57b-e06749127c98",
                ),
                diagnosis_id_one=uuid.UUID(
                    "eea5ca5a-8b43-45fd-8cbd-c6cc1103e759",
                ),
                diagnosis_id_two=uuid.UUID(
                    "5c4aa029-2db9-4202-916e-e93c708f65ff",
                ),
                diagnosis_id_three=uuid.UUID(
                    "81795126-a3ac-443c-b47e-7259a16ab4a2",
                ),
                service_line_era_data=ServiceLineEraData(
                    service_line_adjustments=[
                        ServiceLineAdjustment(
                            created_at=datetime.datetime.fromisoformat(
                                "2023-01-01 00:00:00+00:00",
                            ),
                            adjustment_group_code="CO",
                            adjustment_reason_code="CO",
                            adjustment_amount_cents=1000,
                            adjustment_note="test_note",
                        )
                    ],
                    remittance_advice_remark_codes=["N362"],
                ),
                service_line_manual_adjustments=[
                    ServiceLineAdjustment(
                        created_at=datetime.datetime.fromisoformat(
                            "2023-01-01 00:00:00+00:00",
                        ),
                        adjustment_group_code="CO",
                        adjustment_reason_code="CO",
                        adjustment_amount_cents=1000,
                        adjustment_note="test_note",
                    )
                ],
                related_invoices=[
                    Invoice(
                        id=uuid.UUID(
                            "901be2f1-41bc-456e-9987-4fe2f84f9d75",
                        ),
                        created_at=datetime.datetime.fromisoformat(
                            "2023-01-01 00:00:00+00:00",
                        ),
                        updated_at=datetime.datetime.fromisoformat(
                            "2023-01-01 00:00:00+00:00",
                        ),
                        organzation_id=uuid.UUID(
                            "f13f73d4-4344-46ea-9d93-33bcffbb9f36",
                        ),
                        source_id="9B626577-8808-4F28-9ED1-F0DFF0D49BBC",
                        source_customer_id="624D1972-8C69-4C2F-AEFA-10856F734DB3",
                        patient_external_id="10FED4D6-4C5A-48DF-838A-EEF45A74788D",
                        note="test_note",
                        due_date="2023-10-10",
                        status=InvoiceStatus.DRAFT,
                        url="https://example.com",
                        customer_invoice_url="https://example.com",
                        items=[
                            InvoiceItem(
                                service_line_id=uuid.UUID(
                                    "ced00f23-6e68-4678-9dbc-f5aa2969a565",
                                ),
                                amount_cents=500,
                            )
                        ],
                    )
                ],
                denial_reason=ServiceLineDenialReason(
                    reason=DenialReasonContent.AUTHORIZATION_REQUIRED,
                ),
                place_of_service_code=FacilityTypeCode.PHARMACY,
                service_line_id=uuid.UUID(
                    "ced00f23-6e68-4678-9dbc-f5aa2969a565",
                ),
                procedure_code="99213",
                quantity="1",
                units=ServiceLineUnits.MJ,
                claim_id=uuid.UUID(
                    "026a1fb8-748e-4859-a2d7-3ea9e07d25ae",
                ),
                date_of_service_range=DateRangeOptionalEnd(
                    start_date="2023-01-01",
                    end_date="2023-01-03",
                ),
                date_of_service=datetime.date.fromisoformat(
                    "2023-01-01",
                ),
                end_date_of_service=datetime.date.fromisoformat(
                    "2023-01-03",
                ),
                test_results=[
                    TestResult(
                        result_type=TestResultType.HEMOGLOBIN,
                        value=51.0,
                    ),
                    TestResult(
                        result_type=TestResultType.LDL,
                        value=110.0,
                    ),
                ],
            )
        ],
        eras=[
            Era(
                era_id=uuid.UUID(
                    "4d844ef1-2253-43cd-a4f1-6db7e65cb54b",
                ),
                check_number="CHK12345",
                check_date="2023-10-12",
            )
        ],
    )
    """

    claim_id: ClaimId
    status: ClaimStatus
    clearinghouse: typing.Optional[str] = None
    clearinghouse_claim_id: typing.Optional[str] = None
    payer_claim_id: typing.Optional[str] = None
    clia_number: typing.Optional[str] = None
    service_lines: typing.List[ServiceLine]
    eras: typing.List[Era]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class Race(enum.StrEnum):
    AMERICAN_INDIAN_OR_ALASKA_NATIVE = "AMERICAN_INDIAN_OR_ALASKA_NATIVE"
    WHITE = "WHITE"
    BLACK = "BLACK"
    ASIAN = "ASIAN"
    NATIVE_HAWAIIAN_OR_OTHER_PACIFIC_ISLANDER = "NATIVE_HAWAIIAN_OR_OTHER_PACIFIC_ISLANDER"
    MIDDLE_EASTERN_OR_NORTH_AFRICAN = "MIDDLE_EASTERN_OR_NORTH_AFRICAN"
    OTHER = "OTHER"
    UNKNOWN = "UNKNOWN"
    REFUSED = "REFUSED"
    _UNKNOWN = "__RACE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "Race":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        american_indian_or_alaska_native: typing.Callable[[], T_Result],
        white: typing.Callable[[], T_Result],
        black: typing.Callable[[], T_Result],
        asian: typing.Callable[[], T_Result],
        native_hawaiian_or_other_pacific_islander: typing.Callable[[], T_Result],
        middle_eastern_or_north_african: typing.Callable[[], T_Result],
        other: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        refused: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is Race.AMERICAN_INDIAN_OR_ALASKA_NATIVE:
            return american_indian_or_alaska_native()
        if self is Race.WHITE:
            return white()
        if self is Race.BLACK:
            return black()
        if self is Race.ASIAN:
            return asian()
        if self is Race.NATIVE_HAWAIIAN_OR_OTHER_PACIFIC_ISLANDER:
            return native_hawaiian_or_other_pacific_islander()
        if self is Race.MIDDLE_EASTERN_OR_NORTH_AFRICAN:
            return middle_eastern_or_north_african()
        if self is Race.OTHER:
            return other()
        if self is Race.UNKNOWN:
            return unknown()
        if self is Race.REFUSED:
            return refused()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class TypeOfCareCode(enum.StrEnum):
    C_1 = "1"
    """
    Except Clinics & Special Facilities - Inpatient Part A; Clinics Only - Rural Health Center (RHC); Special Facilities Only - Hospice (non-hospital based)
    """

    C_2 = "2"
    """
    Except Clinics & Special Facilities - Inpatient (Part B) (includes Home Health Agency (HHA) visits under a Part B plan of treatment); Clinics Only - Hospital based or Independent Renal Dialysis Center; Special Facilities Only - Hospice (hospital based)
    """

    C_3 = "3"
    """
    Except Clinics & Special Facilities - Outpatient (includes HHA visits under a Part A plan of treatment and use of HHA DME under a Part A plan of treatment); Special Facilities Only - ASC Services to Hospital Outpatients
    """

    C_4 = "4"
    """
    Except Clinics & Special Facilities - Other (Part B) (includes HHA medical and other health services not under a plan of treatment, SNF diagnostic clinical laboratory services for 'nonpatients,' and referenced diagnostic services); Clinics Only - Other Rehabilitation Facility (ORF); Special Facilities Only - Free Standing Birthing Center
    """

    C_5 = "5"
    """
    Except Clinics & Special Facilities - Intermediate Care - Level I; Clinics Only - Comprehensive Outpatient Rehabilitation Facility (CORF); Special Facilities Only - CAH
    """

    C_6 = "6"
    """
    Except Clinics & Special Facilities - Intermediate Care - Level II; Clinics Only - Community Mental Health Center (CMHC); Special Facilities Only - Residential Facility (not used for Medicare)
    """

    C_7 = "7"
    """
    Except Clinics & Special Facilities - Subacute Inpatient (Revenue Code 019X required) Eight Swing Beds (used to indicate billing for SNF level of care in a hospital with an approved swing bed agreement.); Clinics Only - Free-standing Provider-based Federally Qualified Health Center (FQHC); Special Facilities Only - Reserved for National Assignment
    """

    C_8 = "8"
    """
    Except Clinics & Special Facilities - NA; Clinics Only - Reserved for National Assignment; Special Facilities Only - Reserved for National Assignment
    """

    C_9 = "9"
    """
    Except Clinics & Special Facilities - Reserved for National Assignment; Clinics Only - Other; Special Facilities Only - Other
    """

    _UNKNOWN = "__TYPEOFCARECODE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "TypeOfCareCode":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        c_1: typing.Callable[[], T_Result],
        c_2: typing.Callable[[], T_Result],
        c_3: typing.Callable[[], T_Result],
        c_4: typing.Callable[[], T_Result],
        c_5: typing.Callable[[], T_Result],
        c_6: typing.Callable[[], T_Result],
        c_7: typing.Callable[[], T_Result],
        c_8: typing.Callable[[], T_Result],
        c_9: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is TypeOfCareCode.C_1:
            return c_1()
        if self is TypeOfCareCode.C_2:
            return c_2()
        if self is TypeOfCareCode.C_3:
            return c_3()
        if self is TypeOfCareCode.C_4:
            return c_4()
        if self is TypeOfCareCode.C_5:
            return c_5()
        if self is TypeOfCareCode.C_6:
            return c_6()
        if self is TypeOfCareCode.C_7:
            return c_7()
        if self is TypeOfCareCode.C_8:
            return c_8()
        if self is TypeOfCareCode.C_9:
            return c_9()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .admitting_diagnosis import AdmittingDiagnosis
    from .admitting_diagnosis_code_qualifier import AdmittingDiagnosisCodeQualifier
    from .admitting_diagnosis_new import AdmittingDiagnosisNew
    from .condition_code import ConditionCode
    from .condition_information import ConditionInformation
    from .condition_information_code_qualifier import ConditionInformationCodeQualifier
    from .condition_information_new import ConditionInformationNew
    from .d_8_date import D8Date
    from .diagnosis_related_group import DiagnosisRelatedGroup
    from .diagnosis_related_group_code_qualifier import DiagnosisRelatedGroupCodeQualifier
    from .diagnosis_related_group_new import DiagnosisRelatedGroupNew
    from .external_cause_of_injury import ExternalCauseOfInjury
    from .external_cause_of_injury_code_qualifier import ExternalCauseOfInjuryCodeQualifier
    from .external_cause_of_injury_new import ExternalCauseOfInjuryNew
    from .health_care_code_information import HealthCareCodeInformation
    from .health_care_code_information_create import HealthCareCodeInformationCreate
    from .health_care_code_information_get_all_response import HealthCareCodeInformationGetAllResponse
    from .health_care_code_information_new import HealthCareCodeInformationNew
    from .health_care_code_information_update import HealthCareCodeInformationUpdate
    from .icd_code import IcdCode
    from .msdrg_code import MsdrgCode
    from .occurrence_code import OccurrenceCode
    from .occurrence_information import OccurrenceInformation
    from .occurrence_information_code_qualifier import OccurrenceInformationCodeQualifier
    from .occurrence_information_new import OccurrenceInformationNew
    from .occurrence_span_code import OccurrenceSpanCode
    from .occurrence_span_code_qualifier import OccurrenceSpanCodeQualifier
    from .occurrence_span_information import OccurrenceSpanInformation
    from .occurrence_span_information_new import OccurrenceSpanInformationNew
    from .other_diagnosis_information import OtherDiagnosisInformation
    from .other_diagnosis_information_code_qualifier import OtherDiagnosisInformationCodeQualifier
    from .other_diagnosis_information_new import OtherDiagnosisInformationNew
    from .other_procedure_information import OtherProcedureInformation
    from .other_procedure_information_code_qualifier import OtherProcedureInformationCodeQualifier
    from .other_procedure_information_new import OtherProcedureInformationNew
    from .patient_reason_for_visit import PatientReasonForVisit
    from .patient_reason_for_visit_code_qualifier import PatientReasonForVisitCodeQualifier
    from .patient_reason_for_visit_new import PatientReasonForVisitNew
    from .present_on_admission_indicator_code import PresentOnAdmissionIndicatorCode
    from .principal_diagnosis import PrincipalDiagnosis
    from .principal_diagnosis_code_qualifier import PrincipalDiagnosisCodeQualifier
    from .principal_diagnosis_new import PrincipalDiagnosisNew
    from .principal_procedure_information import PrincipalProcedureInformation
    from .principal_procedure_information_code_qualifier import PrincipalProcedureInformationCodeQualifier
    from .principal_procedure_information_new import PrincipalProcedureInformationNew
    from .rd_8_date import Rd8Date
    from .set_or_clear_admitting_diagnosis import SetOrClearAdmittingDiagnosis
    from .set_or_clear_diagnosis_related_group import SetOrClearDiagnosisRelatedGroup
    from .set_or_clear_principal_diagnosis import SetOrClearPrincipalDiagnosis
    from .set_or_clear_principal_procedure_information import SetOrClearPrincipalProcedureInformation
    from .treatment_code import TreatmentCode
    from .treatment_code_information import TreatmentCodeInformation
    from .treatment_code_information_new import TreatmentCodeInformationNew
    from .treatment_code_qualifier import TreatmentCodeQualifier
    from .value_code import ValueCode
    from .value_code_qualifier import ValueCodeQualifier
    from .value_information import ValueInformation
    from .value_information_new import ValueInformationNew
_dynamic_imports: typing.Dict[str, str] = {
    "AdmittingDiagnosis": ".admitting_diagnosis",
    "AdmittingDiagnosisCodeQualifier": ".admitting_diagnosis_code_qualifier",
    "AdmittingDiagnosisNew": ".admitting_diagnosis_new",
    "ConditionCode": ".condition_code",
    "ConditionInformation": ".condition_information",
    "ConditionInformationCodeQualifier": ".condition_information_code_qualifier",
    "ConditionInformationNew": ".condition_information_new",
    "D8Date": ".d_8_date",
    "DiagnosisRelatedGroup": ".diagnosis_related_group",
    "DiagnosisRelatedGroupCodeQualifier": ".diagnosis_related_group_code_qualifier",
    "DiagnosisRelatedGroupNew": ".diagnosis_related_group_new",
    "ExternalCauseOfInjury": ".external_cause_of_injury",
    "ExternalCauseOfInjuryCodeQualifier": ".external_cause_of_injury_code_qualifier",
    "ExternalCauseOfInjuryNew": ".external_cause_of_injury_new",
    "HealthCareCodeInformation": ".health_care_code_information",
    "HealthCareCodeInformationCreate": ".health_care_code_information_create",
    "HealthCareCodeInformationGetAllResponse": ".health_care_code_information_get_all_response",
    "HealthCareCodeInformationNew": ".health_care_code_information_new",
    "HealthCareCodeInformationUpdate": ".health_care_code_information_update",
    "IcdCode": ".icd_code",
    "MsdrgCode": ".msdrg_code",
    "OccurrenceCode": ".occurrence_code",
    "OccurrenceInformation": ".occurrence_information",
    "OccurrenceInformationCodeQualifier": ".occurrence_information_code_qualifier",
    "OccurrenceInformationNew": ".occurrence_information_new",
    "OccurrenceSpanCode": ".occurrence_span_code",
    "OccurrenceSpanCodeQualifier": ".occurrence_span_code_qualifier",
    "OccurrenceSpanInformation": ".occurrence_span_information",
    "OccurrenceSpanInformationNew": ".occurrence_span_information_new",
    "OtherDiagnosisInformation": ".other_diagnosis_information",
    "OtherDiagnosisInformationCodeQualifier": ".other_diagnosis_information_code_qualifier",
    "OtherDiagnosisInformationNew": ".other_diagnosis_information_new",
    "OtherProcedureInformation": ".other_procedure_information",
    "OtherProcedureInformationCodeQualifier": ".other_procedure_information_code_qualifier",
    "OtherProcedureInformationNew": ".other_procedure_information_new",
    "PatientReasonForVisit": ".patient_reason_for_visit",
    "PatientReasonForVisitCodeQualifier": ".patient_reason_for_visit_code_qualifier",
    "PatientReasonForVisitNew": ".patient_reason_for_visit_new",
    "PresentOnAdmissionIndicatorCode": ".present_on_admission_indicator_code",
    "PrincipalDiagnosis": ".principal_diagnosis",
    "PrincipalDiagnosisCodeQualifier": ".principal_diagnosis_code_qualifier",
    "PrincipalDiagnosisNew": ".principal_diagnosis_new",
    "PrincipalProcedureInformation": ".principal_procedure_information",
    "PrincipalProcedureInformationCodeQualifier": ".principal_procedure_information_code_qualifier",
    "PrincipalProcedureInformationNew": ".principal_procedure_information_new",
    "Rd8Date": ".rd_8_date",
    "SetOrClearAdmittingDiagnosis": ".set_or_clear_admitting_diagnosis",
    "SetOrClearDiagnosisRelatedGroup": ".set_or_clear_diagnosis_related_group",
    "SetOrClearPrincipalDiagnosis": ".set_or_clear_principal_diagnosis",
    "SetOrClearPrincipalProcedureInformation": ".set_or_clear_principal_procedure_information",
    "TreatmentCode": ".treatment_code",
    "TreatmentCodeInformation": ".treatment_code_information",
    "TreatmentCodeInformationNew": ".treatment_code_information_new",
    "TreatmentCodeQualifier": ".treatment_code_qualifier",
    "ValueCode": ".value_code",
    "ValueCodeQualifier": ".value_code_qualifier",
    "ValueInformation": ".value_information",
    "ValueInformationNew": ".value_information_new",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AdmittingDiagnosis",
    "AdmittingDiagnosisCodeQualifier",
    "AdmittingDiagnosisNew",
    "ConditionCode",
    "ConditionInformation",
    "ConditionInformationCodeQualifier",
    "ConditionInformationNew",
    "D8Date",
    "DiagnosisRelatedGroup",
    "DiagnosisRelatedGroupCodeQualifier",
    "DiagnosisRelatedGroupNew",
    "ExternalCauseOfInjury",
    "ExternalCauseOfInjuryCodeQualifier",
    "ExternalCauseOfInjuryNew",
    "HealthCareCodeInformation",
    "HealthCareCodeInformationCreate",
    "HealthCareCodeInformationGetAllResponse",
    "HealthCareCodeInformationNew",
    "HealthCareCodeInformationUpdate",
    "IcdCode",
    "MsdrgCode",
    "OccurrenceCode",
    "OccurrenceInformation",
    "OccurrenceInformationCodeQualifier",
    "OccurrenceInformationNew",
    "OccurrenceSpanCode",
    "OccurrenceSpanCodeQualifier",
    "OccurrenceSpanInformation",
    "OccurrenceSpanInformationNew",
    "OtherDiagnosisInformation",
    "OtherDiagnosisInformationCodeQualifier",
    "OtherDiagnosisInformationNew",
    "OtherProcedureInformation",
    "OtherProcedureInformationCodeQualifier",
    "OtherProcedureInformationNew",
    "PatientReasonForVisit",
    "PatientReasonForVisitCodeQualifier",
    "PatientReasonForVisitNew",
    "PresentOnAdmissionIndicatorCode",
    "PrincipalDiagnosis",
    "PrincipalDiagnosisCodeQualifier",
    "PrincipalDiagnosisNew",
    "PrincipalProcedureInformation",
    "PrincipalProcedureInformationCodeQualifier",
    "PrincipalProcedureInformationNew",
    "Rd8Date",
    "SetOrClearAdmittingDiagnosis",
    "SetOrClearDiagnosisRelatedGroup",
    "SetOrClearPrincipalDiagnosis",
    "SetOrClearPrincipalProcedureInformation",
    "TreatmentCode",
    "TreatmentCodeInformation",
    "TreatmentCodeInformationNew",
    "TreatmentCodeQualifier",
    "ValueCode",
    "ValueCodeQualifier",
    "ValueInformation",
    "ValueInformationNew",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ...commons.types.patient_relationship_to_insured_code_all import PatientRelationshipToInsuredCodeAll
from ...commons.types.street_address_short_zip_optional import StreetAddressShortZipOptional
from .individual_base_optional import IndividualBaseOptional


class SubscriberBaseOptional(IndividualBaseOptional):
    patient_relationship_to_subscriber_code: typing.Optional[PatientRelationshipToInsuredCodeAll] = None
    date_of_birth: typing.Optional[dt.date] = None
    address: typing.Optional[StreetAddressShortZipOptional] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class NameUse(enum.StrEnum):
    USUAL = "USUAL"
    OFFICIAL = "OFFICIAL"
    TEMP = "TEMP"
    NICKNAME = "NICKNAME"
    ANONYMOUS = "ANONYMOUS"
    OLD = "OLD"
    MAIDEN = "MAIDEN"
    _UNKNOWN = "__NAMEUSE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "NameUse":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        usual: typing.Callable[[], T_Result],
        official: typing.Callable[[], T_Result],
        temp: typing.Callable[[], T_Result],
        nickname: typing.Callable[[], T_Result],
        anonymous: typing.Callable[[], T_Result],
        old: typing.Callable[[], T_Result],
        maiden: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is NameUse.USUAL:
            return usual()
        if self is NameUse.OFFICIAL:
            return official()
        if self is NameUse.TEMP:
            return temp()
        if self is NameUse.NICKNAME:
            return nickname()
        if self is NameUse.ANONYMOUS:
            return anonymous()
        if self is NameUse.OLD:
            return old()
        if self is NameUse.MAIDEN:
            return maiden()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        Diagnosis,
        DiagnosisCreate,
        DiagnosisCreateOptional,
        DiagnosisId,
        DiagnosisNotFoundError,
        DiagnosisTypeCode,
        DisallowMultiplePrimaryDiagnosisError,
        ServiceLinesMustHaveAtLeastOneDiagnosisError,
        StandaloneDiagnosisCreate,
    )
    from .errors import (
        DiagnosisNotFoundHttpError,
        DisallowMultiplePrimaryDiagnosisHttpError,
        ServiceLinesMustHaveAtLeastOneDiagnosisHttpError,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "Diagnosis": ".types",
    "DiagnosisCreate": ".types",
    "DiagnosisCreateOptional": ".types",
    "DiagnosisId": ".types",
    "DiagnosisNotFoundError": ".types",
    "DiagnosisNotFoundHttpError": ".errors",
    "DiagnosisTypeCode": ".types",
    "DisallowMultiplePrimaryDiagnosisError": ".types",
    "DisallowMultiplePrimaryDiagnosisHttpError": ".errors",
    "ServiceLinesMustHaveAtLeastOneDiagnosisError": ".types",
    "ServiceLinesMustHaveAtLeastOneDiagnosisHttpError": ".errors",
    "StandaloneDiagnosisCreate": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Diagnosis",
    "DiagnosisCreate",
    "DiagnosisCreateOptional",
    "DiagnosisId",
    "DiagnosisNotFoundError",
    "DiagnosisNotFoundHttpError",
    "DiagnosisTypeCode",
    "DisallowMultiplePrimaryDiagnosisError",
    "DisallowMultiplePrimaryDiagnosisHttpError",
    "ServiceLinesMustHaveAtLeastOneDiagnosisError",
    "ServiceLinesMustHaveAtLeastOneDiagnosisHttpError",
    "StandaloneDiagnosisCreate",
]
